/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: Serial_FLASH.C
	
	Purpose								: This file has the driver functions for reading/writing/erasing/programming LDR 
										  file to serial FLASH device
	
	Author								: Mitesh Moonat & Harshit Gaharwar
	
	VDSP version used for testing		: VisualDSP++ 5.0, Update 10
	
	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369, ADSP-21364,
										  ADSP-21262  EZ-Kits
	
	Connection details 					: See "main.C"
	
	Guidelines for change				: 1) No change required for a different processor.
	
										  2) For a new FLASH device (supporting JEDEC RDID command, following changes are 
										  	 required in "Serial_FLASH_Collect_Device_Info" function:
										     
										     - If the manufacturer ID is not listed in the Switch - Case structure, add 
										       new "case" for the new manufaturer ID. Then add new device specific 
										       information  of the new FLASH device in a new 
										       "if(serial_flash_device_info->IDInfo.sDeviceID==<New Device ID>)" structure.
										       
										     - If the manufacturer ID is already listed in the Switch - Case structure, add 
										       new device specific information  of the new FLASH device in a new 
										       "if(serial_flash_device_info->IDInfo.sDeviceID==<New Device ID>)" structure.
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
*************************************************************************************************************************/
#include "Serial_FLASH.h"


/*************************************************************************************************************************
									Global Variables
**************************************************************************************************************************/
float g_fSPICLK;

SERIAL_FLASH_DEVICE_INFO g_serial_flash_device_info;


/*************************************************************************************************************************
									Function Definitions
**************************************************************************************************************************/

/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Read_Identification
    Description		:   This function reads the manufacturing and device ID information from the FLASH
	Arguments		:	Parameter			 |	Description		 								| Valid values
						DEVICE_INFO			 |  Pointer to the s_id object to be initialized    | unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Serial_FLASH_Read_Identification(s_id* DEVICE_INFO)
{
	//Local variables
	unsigned int uiTxBuff;
	unsigned int uiRxBuff;

	//Init tx buffer with JEDEC RDID instruction (0x9F)
	uiTxBuff=(unsigned int)0x9F<<24; 											

	//Assert the slave select
	SPI_Set_Slave_Select(0);

	//Send SPI_Send_Recieve_32bit command to send 8 bit RDID command and receive 8 bit manufaturer ID and 24 bit device ID
	uiRxBuff = SPI_Send_Receive_32bit(uiTxBuff, true);				

	//Deassert the slave select back
	SPI_Set_Slave_Select(1);

	//Extract the Manufaturer ID and Device ID info
	DEVICE_INFO->cManID = (uiRxBuff>>16) & 0xFF;
	DEVICE_INFO->sDeviceID = uiRxBuff & 0xFFFF;
}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Collect_Device_Info
    Description		:   Collects device info such as manufaturer ID, device ID, sector map, page size, device size etc
	Arguments		:	Parameter				 |	Description		 										| Valid values
						serial_flash_device_info |  Address of the device info object to be initialized		| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Serial_FLASH_Collect_Device_Info(SERIAL_FLASH_DEVICE_INFO* serial_flash_device_info)
{
	//Read device ID and manufaturer ID
	Serial_FLASH_Read_Identification(&(serial_flash_device_info->IDInfo));

	//Manufaturer name and Device name
	switch(serial_flash_device_info->IDInfo.cManID)
	{
		//Numonyx
		case 0x20:
				//Initialize the manufaturer name here
				strcpy(serial_flash_device_info->ManName,"Numonyx");

				//Compare the device ID to check if the device is M25P16 - 214xx EZ-Kits
				if(serial_flash_device_info->IDInfo.sDeviceID==0x2015)
				{
					//Initialize the complete device name here
					strcpy(serial_flash_device_info->DeviceName,"M25P16 - 16 Mbit, Low Voltage, Serial Flash Memory with 75 MHz SPI Bus Interface");

					//Initialize the sector map info here
					serial_flash_device_info->SectorInfo.uiBytesPerPage=		256;
					serial_flash_device_info->SectorInfo.uiPagesPerSector=		256;
					serial_flash_device_info->SectorInfo.uiTotalSectors=		32;
					serial_flash_device_info->SectorInfo.uiTotalBytes=			2097152;
				}
				
				//Compare the device ID to check if the device is M25P20 - 21364/21369 EZ-Kits
				else if(serial_flash_device_info->IDInfo.sDeviceID==0x2012)
				{
					//Initialize the complete device name here
					strcpy(serial_flash_device_info->DeviceName,"M25P20 - 2 Mbit, Low Voltage, Serial Flash Memory with 75 MHz SPI Bus Interface");

					//Initialize the sector map info here
					serial_flash_device_info->SectorInfo.uiBytesPerPage=		256;
					serial_flash_device_info->SectorInfo.uiPagesPerSector=		256;
					serial_flash_device_info->SectorInfo.uiTotalSectors=		4;
					serial_flash_device_info->SectorInfo.uiTotalBytes=			262144;

				}
	
				//Initialize the command set info here - common for same different FLASH devices from the same manufaturer 
				serial_flash_device_info->CommandSet.WREN 		=	0x06;		   //Write Enable
				serial_flash_device_info->CommandSet.RDSR 		=	0x05;		   //Read Status Register
				serial_flash_device_info->CommandSet.READ 		=	0x03;		   //Read Data Bytes (f <= 20 MHz)
				serial_flash_device_info->CommandSet.PP	 		=	0x02;		   //Page Program
				serial_flash_device_info->CommandSet.SE   		=	0xD8;		   //Sector Erase
				
				break;

		//Windbond
		case 0xEF:
				//Initialize the manufaturer name here
				strcpy(serial_flash_device_info->ManName,"Windbond");

				//Compare the device ID to check if the device is W25X40BV - External
				if(serial_flash_device_info->IDInfo.sDeviceID==0x3013)
				{
					//Initialize the complete device name here
					strcpy(serial_flash_device_info->DeviceName,"W25X40BV - 4 Mbit Serial FLASH Memory With 4 KB Sectors and Dual I/O SPI");

					//Initialize the sector map info here
					serial_flash_device_info->SectorInfo.uiBytesPerPage=		256;
					serial_flash_device_info->SectorInfo.uiPagesPerSector=		16;
					serial_flash_device_info->SectorInfo.uiTotalSectors=		128;
					serial_flash_device_info->SectorInfo.uiTotalBytes=			524288;
				}

				//Initialize the command set info here - common for same different FLASH devices from the same manufaturer 
				serial_flash_device_info->CommandSet.WREN 		=	0x06;		   //Write Enable
				serial_flash_device_info->CommandSet.RDSR 		=	0x05;		   //Read Status Register
				serial_flash_device_info->CommandSet.READ 		=	0x03;		   //Read Data Bytes (f <= 20 MHz)
				serial_flash_device_info->CommandSet.PP	 		=	0x02;		   //Page Program
				serial_flash_device_info->CommandSet.SE   		=	0x20;		   //Sector Erase
				
				break;
	
		//If no match found for device and(or) manufaturer ID, initialize the device info to default as defined in "Serial_FALSH.h"			
		default:
		
				//Printf can't be used for 2137x because of less memory
				#if !defined(__2137x__)
					printf("\nManufaturer ID and (or) Device ID match not found in the library, assigning the default values\n");
				#endif
				
				//Initialize the default manufaturer name here
				strcpy(serial_flash_device_info->ManName,MANUFATURER_DEFAULT);

				//Initialize the default complete device name here
				strcpy(serial_flash_device_info->DeviceName,DEVICE_NAME_DEFAULT);
				
				//Initialize the default sector map info here
				serial_flash_device_info->SectorInfo.uiBytesPerPage=		BYTES_PER_PAGE_DEFAULT;
				serial_flash_device_info->SectorInfo.uiPagesPerSector=		PAGES_PER_SECTOR_DEFAULT;
				serial_flash_device_info->SectorInfo.uiTotalSectors=		TOTAL_SECTORS_DEFAULT;
				serial_flash_device_info->SectorInfo.uiTotalBytes=			TOTAL_BYTES_DEFAULT;

				//Initialize the default command set info here 
				serial_flash_device_info->CommandSet.WREN 		=	WREN_DEFAULT;		   //Write Enable
				serial_flash_device_info->CommandSet.RDSR 		=	RDSR_DEFAULT;		   //Read Status Register
				serial_flash_device_info->CommandSet.READ 		=	READ_DEFAULT;		   //Read Data Bytes (f <= 20 MHz)
				serial_flash_device_info->CommandSet.PP	 		=	PP_DEFAULT;		   //Page Program
				serial_flash_device_info->CommandSet.SE   		=	SE_DEFAULT;		   //Sector Erase
				
				break;
	}
}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Print_Device_Info
    Description		:   Prints device info such as manufaturer ID, device ID, sector map, page size, device size etc
	Arguments		:	Parameter				 |	Description		 										| Valid values
						serial_flash_device_info |  Address of the device info object to be printed			| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Serial_FLASH_Print_Device_Info(SERIAL_FLASH_DEVICE_INFO* serial_flash_device_info)
{

	#if !defined(__2137x__)
	
		printf("\n***************************************************************************************************************");
		printf("\n						 				 Serial FLASH Device Information							 							 ");
		printf("\n***************************************************************************************************************");
		printf("\n");
		printf("\nManufacturer		: %s",serial_flash_device_info->ManName);
		printf("\nDevice name			: %s",serial_flash_device_info->DeviceName);
		printf("\nBytes per page		: %d",serial_flash_device_info->SectorInfo.uiBytesPerPage);
		printf("\nPages per sector	: %d",serial_flash_device_info->SectorInfo.uiPagesPerSector);
		printf("\nTotal sectors		: %d",serial_flash_device_info->SectorInfo.uiTotalSectors);
		printf("\nTotal bytes			: %d",serial_flash_device_info->SectorInfo.uiTotalBytes);
		printf("\n");
		printf("\n***************************************************************************************************************");
		printf("\n");
		
	#endif

}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Read_Status
    Description		:   This function reads the status regsiter of the SPI FLASH
	Arguments		:	Parameter			 |	Description		 					| Valid values
						Serial_FLASH_Info	 |  Pointer to the device info object 	| unsigned int 32 bit
	Return value	:   Returns the content of the status register
**************************************************************************************************************************/
unsigned char Serial_FLASH_Read_Status(SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{
	//Local variables
	short uiTxBuff;
	short uiRxBuff;
	unsigned char uiStatusRegister;

	//Initialize TX buffer with RDSR command
	uiTxBuff = Serial_FLASH_Info->CommandSet.RDSR<<8;												

	//Assert the slave select 
	SPI_Set_Slave_Select(0);

	//Call SPI_Send_Receive_16bit to send 8bit command and receive 8 bit status
	uiRxBuff = SPI_Send_Receive_16bit(uiTxBuff, true);				

	//Deassert the slave select 
	SPI_Set_Slave_Select(1);

	//Extract the 8 bit status content from the RX buffer
	uiStatusRegister = (unsigned char) (uiRxBuff&0xFF); 

	return uiStatusRegister;
}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Write_Enable
    Description		:   This function sets the write enable latch (WEL) bit of the status register and has to be called
    					before each program/erase function.
	Arguments		:	Parameter			 |	Description		 					|	Valid values
						Serial_FLASH_Info	 |  Pointer to the device info object 	| 	unsigned int 32 bit
	Return value	:   Returnss 1 if write enable was successfull else returns 0
**************************************************************************************************************************/
bool Serial_FLASH_Write_Enable(SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{
	//Local variables
	unsigned char uiTxBuff;
	unsigned char uiStatusRegister;
	bool bStatus;

	//Initialize TX buffer with WREN command
	uiTxBuff=Serial_FLASH_Info->CommandSet.WREN;													

	//Assert the slave select 
	SPI_Set_Slave_Select(0);

	//Call SPI_Send_Receive_8bit to send 8 bit WREN command
	SPI_Send_8bit(uiTxBuff, true);						

	//Deassert the slave select
	SPI_Set_Slave_Select(1);

	//Read the status register
	uiStatusRegister=Serial_FLASH_Read_Status(Serial_FLASH_Info);

	//Check if the WEL bit is set as expected, if not report error
	if((uiStatusRegister&WEL)==0)
		return 0;
	else
		return 1;

}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Sector_Erase
    Description		:   This function erases a particular sector of the FLASH
	Arguments		:	Parameter			 |	Description		 								| Valid values
						sector_address		 |  Any byte address belonging to the sector	    | unsigned int 32 bit
						Serial_FLASH_Info	 |  Pointer to the device info object 				| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Serial_FLASH_Sector_Erase(unsigned int sector_address, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{

	//Local variables
	unsigned char uiTxBuff;
	unsigned char uiStatusRegister;

	//Call Serial_FLASH_Write_Enable to set the WEL bit
	Serial_FLASH_Write_Enable(Serial_FLASH_Info);

	//Init Tx buffer with SE instruction and 24 bit address
	uiTxBuff=sector_address;
	uiTxBuff|=Serial_FLASH_Info->CommandSet.SE<<24;

	//Assert slave select
	SPI_Set_Slave_Select(0);

	//Call SPI_Send_32bit to send 8 bit SE command with 24 bit sector address
	SPI_Send_32bit(uiTxBuff, true);							

	//Deassert the slave select 
	SPI_Set_Slave_Select(1);

	//Wait till erase is complete by reading the status register
	do
	{
		uiStatusRegister=Serial_FLASH_Read_Status(Serial_FLASH_Info);

	}
	while((uiStatusRegister&WIP)!=0);

}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Read_Byte
    Description		:   This function reads a single byte from a particular FLASH address
	Arguments		:	Parameter			 |	Description		 								| Valid values
						uiAddress			 |  Address of the byte to be read from 			| unsigned int 32 bit
						Serial_FLASH_Info	 |  Pointer to the device info object 				| unsigned int 32 bit
	Return value	:   Byte read from the address
**************************************************************************************************************************/
unsigned char  Serial_FLASH_Read_Byte(unsigned int uiAddress, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{
	//Local variables
	unsigned char uiTxBuff;
	unsigned char uiRxBuff;

	//Init Tx buffer with READ instruction and 24 bit address
	uiTxBuff=uiAddress;
	uiTxBuff|=Serial_FLASH_Info->CommandSet.READ<<24;

	//Assert the slave select
	SPI_Set_Slave_Select(0);

	//Send 8 bit READ command with 24 bit read address
	SPI_Send_32bit(uiTxBuff, true);

	//Receive N 8 bit words from the FLASH
	SPI_Receive_Multiple_8bit(&uiRxBuff, 1, bMSBF);

	//Deassert the slave select
	SPI_Set_Slave_Select(1);

	return uiRxBuff;
}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Read_Bytes
    Description		:   This function writes a series of bytes to a particular FLASH location
	Arguments		:	Parameter			 |	Description		 										| Valid values
						ucBuff				 |  Start address of the buffer to be readinto 				| unsigned int 32 bit
						uiStartAddress		 |  Start byte address of the FLASH location		 		| unsigned int 32 bit
						uiLength			 |  Total number of bytes to be written					    | unsigned int 32 bit
						bMSBF				 |  Whether the data should be written as MSBF or LSBF 		| =true for MSBF and = false for LSBF
						Serial_FLASH_Info	 |  Pointer to the device info object 						| unsigned int 32 bit
	Return value	:   =1 if success = 0 if fail
**************************************************************************************************************************/
void Serial_FLASH_Read_Bytes(unsigned char* cBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{
	//Local variables
	unsigned char uiTxBuff;
	unsigned char uiRxBuff;

	//Init Tx buffer with READ instruction and 24 bit address
	uiTxBuff=uiStartAddress;
	uiTxBuff|=Serial_FLASH_Info->CommandSet.READ<<24;
	
	//Assert the slave select
	SPI_Set_Slave_Select(0);

	//Send 8 bit READ command with 24 bit read address
	SPI_Send_32bit(uiTxBuff, true);

	//Receive one 8 bit words from the FLASH
	SPI_Receive_Multiple_8bit(cBuff, uiLength, bMSBF);

	//Deassert the slave select
	SPI_Set_Slave_Select(1);
}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Page_Program
    Description		:   This function programs a page of the FLASH
	Arguments		:	Parameter			 |	Description		 												| Valid values
						ucBuff				 |  Pointer to the buffer to be written to the FLASH 				| unsigned int 32 bit
						uiStartAddress		 |  Start Byte address corresponding to the page to be progammed	| unsigned int 32 bit
						uiLength			 |  No of words to be written to the page 							| 1-256
						bMSBF				 |  Whether the data to be sent to the FLASH as LSBF or MSBF		| =true for MSBF and = false for LSBF
						Serial_FLASH_Info	 |  Pointer to the device info object 								| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Serial_FLASH_Page_Program(unsigned char *ucBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info )
{

	//Local variables
	unsigned char uiTxBuff;
	unsigned char uiRxBuff;
	unsigned char uiStatusRegister;

	//Init Tx buffer with PP instruction and 24 bit address
	uiTxBuff=uiStartAddress;
	uiTxBuff|=Serial_FLASH_Info->CommandSet.PP<<24;

	//Assert the slave select
	SPI_Set_Slave_Select(0);

	//Send 8 bit PP command with 24 bit address
	SPI_Send_32bit(uiTxBuff, true);

	//Send N 8 bit words to be programmed
	SPI_Send_Multiple_8bit(ucBuff, uiLength, bMSBF);

	//Deassert slave select
	SPI_Set_Slave_Select(1);

	//Wait till write is complete by reading the status register
	do
	{
		uiStatusRegister=Serial_FLASH_Read_Status(Serial_FLASH_Info);

	}
	while((uiStatusRegister&WIP)!=0);

}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Write_Byte
    Description		:   This function writes a byte to a particular FLASH location
	Arguments		:	Parameter			 |	Description		 										| Valid values
						uiAddress			 |  Address of the byte location to be writte 				| unsigned int 32 bit
						bMSBF				 |  Whether the data should be written as MSBF or LSBF 		| =true for MSBF and = false for LSBF
						Serial_FLASH_Info	 |  Pointer to the device info object 						| unsigned int 32 bit
	Return value	:   =1 if success = 0 if fail
**************************************************************************************************************************/
void Serial_FLASH_Write_Byte(unsigned int uiAddress, unsigned char data, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info )
{
	//Enable write
	Serial_FLASH_Write_Enable(Serial_FLASH_Info);

	//Page Program
	Serial_FLASH_Page_Program(&data, uiAddress, 1, bMSBF, Serial_FLASH_Info );

}


/**************************************************************************************************************************
	Function name	:   Serial_FLASH_Write_Bytes
    Description		:   This function writes a series of bytes to a particular FLASH location
	Arguments		:	Parameter			 |	Description		 										| Valid values
						ucBuff				 |  Start address of the buffer to be written 				| unsigned int 32 bit
						uiStartAddress		 |  Start byte address of the FLASH location		 		| unsigned int 32 bit
						uiLength			 |  Total number of bytes to be written					    | unsigned int 32 bit
						bMSBF				 |  Whether the data should be written as MSBF or LSBF 		| =true for MSBF and = false for LSBF
						Serial_FLASH_Info	 |  Pointer to the device info object 						| unsigned int 32 bit
	Return value	:   =1 if success = 0 if fail
**************************************************************************************************************************/
void Serial_FLASH_Write_Bytes(unsigned char* ucBuff, unsigned int uiStartAddress, unsigned int uiLength, bool bMSBF, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{

	//Local variables
	unsigned int uiStartLength;				//Holds the number of  bytes which can be programmed in the first page
	unsigned int uiStartPageStartAddress;	//Holds the start address ofthe starting page
	unsigned int uiRemainingLength;			//Holds the number of bytes still remaining to be programmed
	unsigned int uiCurrentLength;			//Holds the number of bytes which has be programmed in the current page
	unsigned int uiPageSize;				//Holds the size of a page in bytes (usually  256)
	unsigned char* ucTemp;					//Temporary char variable pointer
	int i;									//For loop iterator
	
	//Initialize the temp pointer to buffer start address
	ucTemp=ucBuff;

	//Initialize the page size	
	uiPageSize=g_serial_flash_device_info.SectorInfo.uiBytesPerPage;

	//Calculate the number of bytes to be programmed in the start page
	uiStartLength=uiPageSize-uiStartAddress%uiPageSize;

	//Calculate the start address ofthe starting page
	uiStartPageStartAddress=((floor)(uiStartAddress/uiPageSize))*uiPageSize;

	//Initialize the remaining length to total length in the begining
	uiRemainingLength=uiLength;

	i=0;

	while(1)
	{
		//Check if it is the first iteration
		if(i==0)
		{
			//Check if number of bytes remaining to be programmed can all accomodate in the current page
			if(uiRemainingLength>=uiStartLength)
			{
				//If no, initialize current length to the number of bytes which can be accomodated in the start page
				uiCurrentLength = uiStartLength;		
				
				//Decrement the remaining length by current length
				uiRemainingLength-=uiCurrentLength;
			}
			else
			{
				//If yes, initialize current length to remaining bytes
				uiCurrentLength = uiRemainingLength;
				
				//No more remaining bytes
				uiRemainingLength=0;
			}
			
			//Send Write Enable command to allow programming		
			Serial_FLASH_Write_Enable(Serial_FLASH_Info);
			
			//Send Page Program command to program current length bytes to the current page
			Serial_FLASH_Page_Program(ucTemp, uiStartAddress, uiCurrentLength, bMSBF, Serial_FLASH_Info );

			//Increment the buffer read pointer by current length
			ucTemp+=uiCurrentLength;

		}

		//If not first iteration
		else  
		{
			//If all bytes have been written already RETURN
			if(uiRemainingLength==0)
				break;
			
			//Otherwise continue
			else
			{
				//Check if number of bytes remaining to be programmed can all accomodate in the current page
				if(uiRemainingLength>=uiPageSize)
				{
					//If no, initialize current length to the page size 
					uiCurrentLength = uiPageSize;
					
					//Decrement the remaining length by current length
					uiRemainingLength-=uiCurrentLength;
				}
				else
				{
					//If yes, initialize current length to remaining bytes
					uiCurrentLength = uiRemainingLength;
					
					//No more remaining bytes
					uiRemainingLength=0;
				}

				//Send Write Enable command to allow programming	
				Serial_FLASH_Write_Enable(Serial_FLASH_Info);
				
				//Send Page Program command to program current length bytes to the current page
				Serial_FLASH_Page_Program(ucTemp, (uiStartPageStartAddress+i*uiPageSize), uiCurrentLength, bMSBF, Serial_FLASH_Info );

				//Increment the buffer read pointer by current length
				ucTemp+=uiCurrentLength;
			}
		}

		//Increment the loop counter
		i++;
	}
}


/**************************************************************************************************************************
	Function name	:   Program_LDR_File_to_Serial_FLASH
    Description		:   Programs ldr file contents ASCII format to the serial FLASH 
	Arguments		:	Parameter				 |	Description		 										| Valid values
						sLDRFilePath			 |  Absolute path of the ldr file							| String
						uiStartAddress			 |  Start byte offset										| Unsigned int 32 bits
						Serial_FLASH_Info		 |  Pointer to the Serial FLASH info information			| Unsigned int 32 bits
	Return value	:   -1 if ldr file coudn't be opened, -2 if the verification failed, 1 if everything passed 
**************************************************************************************************************************/
#if !defined(__2137x__)
int Program_LDR_File_to_Serial_FLASH(char* sLDRFilePath, unsigned int uiStartAddress, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{

	//Local variables
	FILE* fp;						//Handle for the ldr file
	unsigned char* ucpTxBuff;		//Pointer to the temporary buffer to store the ldr data	
	unsigned char* ucpVerifyBuff;	//Pointer to the verification buffer
	int i;							//For loop iteration
	unsigned char* ucpTemp;			//Temporary pointers
	unsigned char* ucpTemp1;	  
	int iStatus;					//Temporary variable to hold the status 
	unsigned char ucTemp;			//Temp char variable
	int iInnerCount;				//Holds the number of words to be progammed for each programming iteration
	int iOuterCount;				//Holds the current value of total bytes already programmed
	unsigned int uiBytesPerSector;	//Holds the sector size info locally
	
	//Initialize the sector size from the serial flash info
	uiBytesPerSector=(Serial_FLASH_Info->SectorInfo.uiBytesPerPage)*(Serial_FLASH_Info->SectorInfo.uiPagesPerSector);
	
	//Allocating the temporary buffer size
	ucpTxBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpTxBuff));					
	
	//Allocating the temporary buffer size
	ucpVerifyBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpVerifyBuff));			
			
	//Open the ldr file in read mode
	fp=fopen(sLDRFilePath, "r");																
	
	//Return -1 if the file couldn't be opened			
	if(fp==NULL)																				
		return -1;
		
	//Initialize outer count to 0
	iOuterCount=0;
	
	//Read the first byte
	iStatus=fscanf(fp,"%x",&ucTemp);
	

	//Program the ldr file now, continue till end of file
	while(iStatus!=EOF)
	{
		ucpTemp=ucpTxBuff;	//Reinitialize the pointer to the start of the temp buffer
	
		//Initialize inner count back to 0
		iInnerCount=0; 	
		
		//Initialize the temporary buffer here
		while((iStatus!=EOF)&&(iInnerCount<LDR_TEMP_BUFF_SIZE))
		{
			*ucpTemp++=ucTemp;
			iInnerCount++; 
			iStatus=fscanf(fp,"%x",&ucTemp);
			
		}		
		
		//For first time programming
		if(iOuterCount==0)
		{
			//Erase initial affected sector 
			Serial_FLASH_Sector_Erase(uiStartAddress+0,Serial_FLASH_Info);	
			
			//Erase the next affected sector 
			if(((iOuterCount+uiStartAddress-1)/uiBytesPerSector)!=((iOuterCount+uiStartAddress+iInnerCount-1)/uiBytesPerSector)) //Check for the next sector boundary
				Serial_FLASH_Sector_Erase(iOuterCount+uiStartAddress+iInnerCount-1, Serial_FLASH_Info);	
	
			//Write the next iInnterCount bytes using "Serial FLASH Write Bytes" function
			Serial_FLASH_Write_Bytes(ucpTxBuff, iOuterCount+uiStartAddress,iInnerCount, false, Serial_FLASH_Info);		//Write the next iInnterCount bytes
		
			//Read the iInnterCount bytes written above using "Serial FLASH Read Bytes" function
			Serial_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount+uiStartAddress,iInnerCount, false, Serial_FLASH_Info);	//Read the iInnerCount bytes
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				//If end of file, return 1
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
		else
		{
			//Erase the next affected sector 
			if(((uiStartAddress+iOuterCount-1)/uiBytesPerSector)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBytesPerSector)) //Check for the next sector boundary
					Serial_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1, Serial_FLASH_Info);	//Erase the next sector 
		
			//Write the next iInnterCount bytes using "Serial FLASH Write Bytes" function
			Serial_FLASH_Write_Bytes(ucpTxBuff, iOuterCount,iInnerCount, false, Serial_FLASH_Info);
		
			//Read the iInnterCount bytes written above using "Serial FLASH Read Bytes" function
			Serial_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount,iInnerCount, false, Serial_FLASH_Info);
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				//If end of file, return 1
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
		}
	}
	return 1;
}
#endif

/**************************************************************************************************************************
	Function name	:   Program_LDR_File_to_Serial_FLASH_From_Memory
    Description		:   Programs ldr file contents stored statically in internal/external memory to the serial FLASH 
	Arguments		:	Parameter				 |	Description		 										| Valid values
						ucpSourceBuff			 |  Start address of the source buffer						| Unsigned int 32 bits
						uiLdrLength				 |  Total size of ldr										| Unsigned int 32 bits
						uiStartAddress			 |  Start byte offset										| Unsigned int 32 bits
						Serial_FLASH_Info		 |  Pointer to the Serial FLASH info information			| Unsigned int 32 bits
	Return value	:   -1 if ldr file coudn't be opened, -2 if the verification failed, 1 if everything passed 
**************************************************************************************************************************/
int Program_LDR_File_to_Serial_FLASH_From_Memory(unsigned char* ucpSourceBuff, unsigned int uiLdrLength, unsigned int uiStartAddress, SERIAL_FLASH_DEVICE_INFO *Serial_FLASH_Info)
{

	//Local variables
	unsigned char* ucpTxBuff;		//Pointer to the temporary buffer to store the ldr data	
	unsigned char* ucpVerifyBuff;	//Pointer to the verification buffer
	int i;							//For loop iteration
	unsigned char* ucpTemp;			//Temporary pointers
	unsigned char* ucpTempSource;
	unsigned char* ucpTemp1;																		
	int iStatus;					//Temporary variable to hold the status 		
	unsigned char ucTemp;			//Temp char variable
	int iInnerCount;				//Holds the number of words to be progammed for each programming iteration
	int iOuterCount;				//Holds the current value of total bytes already programmed
	unsigned int uiBytesPerSector;	//Holds the sector size info locally
	
	//Initialize the sector size from the serial flash info
	uiBytesPerSector=(Serial_FLASH_Info->SectorInfo.uiBytesPerPage)*(Serial_FLASH_Info->SectorInfo.uiPagesPerSector);
	
	//Allocating the temporary buffer size
	ucpTxBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpTxBuff));					
	
	//Allocating the temporary buffer size
	ucpVerifyBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpVerifyBuff));			

	//Initialize temp source pointer with the start address of the ldr buffer	
	ucpTempSource = ucpSourceBuff;
	
	//Initialize outer count to 0
	iOuterCount=0;
	
	//Program the ldr file now
	while(iOuterCount<uiLdrLength)
	{
		//Reinitialize the pointer to the start of the temp buffer
		ucpTemp=ucpTxBuff;
	
		//Initialize inner count back to 0
		iInnerCount=0; 	
		
		//Initialize the temporary buffer here
		while((iInnerCount<LDR_TEMP_BUFF_SIZE)&&((iOuterCount+iInnerCount)<uiLdrLength))
		{
			*ucpTemp++=*ucpTempSource++;
			iInnerCount++; 
		}		
		
		//For first time programming
		if(iOuterCount==0)
		{
			//Erase initial affected sector 
			Serial_FLASH_Sector_Erase(uiStartAddress+0, Serial_FLASH_Info);	
			
			//Check for the next sector boundary and erase the next sector if required
			if(((iOuterCount+uiStartAddress-1)/uiBytesPerSector)!=((iOuterCount+uiStartAddress+iInnerCount-1)/uiBytesPerSector)) 
				Serial_FLASH_Sector_Erase(iOuterCount+uiStartAddress+iInnerCount-1, Serial_FLASH_Info);	
	
			//Write the next iInnterCount bytes using "Serial FLASH Write Bytes" function
			Serial_FLASH_Write_Bytes(ucpTxBuff, iOuterCount+uiStartAddress,iInnerCount, false, Serial_FLASH_Info );		//Write the next iInnterCount bytes
			
			//Read the iInnterCount bytes written above using "Serial FLASH Read Bytes" function
			Serial_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount+uiStartAddress,iInnerCount, false, Serial_FLASH_Info);	//Read the iInnerCount bytes
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of ldr buff
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				//If end of file, return 1
			{
					//Free the buffers and return if end of ldr buff
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
		else
		{
			//Check for the next sector boundary and erase the next sector if required
			if(((uiStartAddress+iOuterCount-1)/uiBytesPerSector)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBytesPerSector)) 
					Serial_FLASH_Sector_Erase(iOuterCount+iInnerCount-1, Serial_FLASH_Info);	
		
			//Write the next iInnterCount bytes using "Serial FLASH Write Bytes" function
			Serial_FLASH_Write_Bytes(ucpTxBuff, iOuterCount,iInnerCount, false, Serial_FLASH_Info);
		
			//Read the iInnterCount bytes written above using "Serial FLASH Read Bytes" function
			Serial_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount,iInnerCount, false, Serial_FLASH_Info);
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of ldr buff
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				//If end of file, return 1
			{
					//Free the buffers and return if end of ldr buff
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
	}
	
	return 1;

}

/**************************************** END OF FILE*********************************************************************/
